@@ -1,6 +1,6 @@
 """
 PyDetex
-https://github.com/ppizarror/pydetex
+https://github.com/ppizarror/PyDetex
 
 GUI SETTINGS
 Provides settings for the gui.
@@ -103,9 +103,14 @@ class _LangManager(object):
                 'menu_copy': 'Copy',
                 'menu_cut': 'Cut',
                 'menu_paste': 'Paste',
+                'open_file': 'Open file',
+                'open_file_latex_file': 'LaTeX file',
+                'open_file_select': 'Select a LaTeX file',
                 'pipeline_simple': 'Simple',
+                'pipeline_simple_description': 'Removes common Tex commands, replaces cites and references',
                 'pipeline_strict': 'Strict',
-                'placeholder': 'Write or paste here your \\texttt{LaTeX} code. It simply removes all tex-things, and returns a nice plain text!',
+                'pipeline_strict_description': 'An extension of the Simple pipeline which removes all Tex commands and environments',
+                'placeholder': 'Write or paste here your \\texttt{LaTeX} code. It simply removes all tex-things and returns a friendly plain text!',
                 'process': 'Process',
                 'process_clip': 'Process from clipboard',
                 'process_copy': 'Copy to clipboard',
@@ -115,6 +120,7 @@ class _LangManager(object):
                 'status_cursor': 'Cursor: {0}:{1}',
                 'status_cursor_input_focusout': 'Input not selected',
                 'status_cursor_input_focusout_min': 'Not selected',
+                'status_cursor_input_focusout_min2': 'No sel',
                 'status_cursor_min': 'Cur: {0}:{1}',
                 'status_cursor_null': 'Cursor: Empty',
                 'status_cursor_selected': 'Selected',
@@ -125,6 +131,7 @@ class _LangManager(object):
                 'status_cursor_selected_min': 'Sel',
                 'status_idle': 'Idle',
                 'status_processing': 'Processing',
+                'status_requesting_file': 'Requesting file',
                 'status_words': 'Words: {0}',
                 'status_writing': 'Writing',
                 'tag_repeated': 'repeated',
@@ -188,8 +195,13 @@ class _LangManager(object):
                 'menu_copy': 'Copiar',
                 'menu_cut': 'Cortar',
                 'menu_paste': 'Pegar',
+                'open_file': 'Abrir archivo',
+                'open_file_latex_file': 'Archivo LaTeX',
+                'open_file_select': 'Selecciona un archivo LaTeX',
                 'pipeline_simple': 'Simple',
+                'pipeline_simple_description': 'Elimina comandos Tex comunes, remplaza citas y referencias',
                 'pipeline_strict': 'Estricto',
+                'pipeline_strict_description': 'Una extensión del pipeline simple que elimina todos los entornos y comandos',
                 'placeholder': 'Escribe o pega aquí tu código \\texttt{LaTeX}. El programa simplemente eliminará todo lo relacionado a tex y retornará un lindo texto plano!',
                 'process': 'Procesar',
                 'process_clip': 'Procesar desde portapapeles',
@@ -200,6 +212,7 @@ class _LangManager(object):
                 'status_cursor': 'Cursor: {0}:{1}',
                 'status_cursor_input_focusout': 'Texto entrada no seleccionado',
                 'status_cursor_input_focusout_min': 'No seleccionado',
+                'status_cursor_input_focusout_min2': 'No sel',
                 'status_cursor_min': 'Cur: {0}:{1}',
                 'status_cursor_null': 'Cursor: Vacío',
                 'status_cursor_selected': 'Selección',
@@ -210,6 +223,7 @@ class _LangManager(object):
                 'status_cursor_selected_min': 'Sel',
                 'status_idle': 'Esperando',
                 'status_processing': 'Procesando',
+                'status_requesting_file': 'Esperando archivo',
                 'status_words': 'Palabras: {0}',
                 'status_writing': 'Escribiendo',
                 'tag_repeated': 'repetido',
@@ -222,14 +236,7 @@ class _LangManager(object):
         }
 
         # Extend languages if not defined
-        for k in self._lang.keys():
-            if k == 'en':
-                continue
-            for t in self._lang['en'].keys():
-                if t not in self._lang[k]:
-                    error = f'Language entry "{t}" on lang "{k}" does not exist'
-                    warn(error)
-                    self._lang[k][t] = self._lang['en'][t]
+        ut.complete_langs_dict(self._lang)
 
     def get_available(self) -> List[str]:
         """
@@ -301,8 +308,9 @@ class Settings(object):
         self.CFG_CHECK_REPETITION = 'CHECK_REPETITION'
         self.CFG_FONT_SIZE = 'FONT_SIZE'
         self.CFG_LANG = 'LANG'
-        self.CFG_OUTPUT_FONT_FORMAT = 'OUTPUT_FONT_FORMAT'
         self.CFG_LAST_OPENED_DAY = 'LAST_OPENED_DAY'
+        self.CFG_LAST_OPENED_FOLDER = 'LAST_OPENED_FOLDER'
+        self.CFG_OUTPUT_FONT_FORMAT = 'OUTPUT_FONT_FORMAT'
         self.CFG_PIPELINE = 'PIPELINE'
         self.CFG_WINDOW_SIZE = 'WINDOW_SIZE'
 
@@ -327,6 +335,7 @@ class Settings(object):
             self.CFG_FONT_SIZE: (11, int, self._valid_font_sizes),
             self.CFG_LANG: ('en', str, self._lang.get_available()),
             self.CFG_LAST_OPENED_DAY: (ut.get_number_of_day(), int, lambda x: x >= 0),
+            self.CFG_LAST_OPENED_FOLDER: ('/', str, lambda x: os.path.isdir(x)),
             self.CFG_OUTPUT_FONT_FORMAT: (True, bool, [True, False]),
             self.CFG_PIPELINE: (self._available_pipelines[0], str, self._available_pipelines),
             self.CFG_REPETITION_DISTANCE: (15, int, lambda x: 50 > x > 1),
@@ -336,7 +345,7 @@ class Settings(object):
             self.CFG_REPETITION_USE_STOPWORDS: (True, bool, [True, False]),
             self.CFG_TOTAL_OPENED_APP: (0, int, lambda x: x >= 0),
             self.CFG_TOTAL_PROCESSED_WORDS: (0, int, lambda x: x >= 0),
-            self.CFG_WINDOW_SIZE: (self._valid_window_sizes[0], str, self._valid_window_sizes)
+            self.CFG_WINDOW_SIZE: (self._valid_window_sizes[1], str, self._valid_window_sizes)
         }
 
         # The valid settings
