@@ -17,12 +17,13 @@ __all__ = [
 ]
 
 import concurrent.futures
+import textwrap
 import tkinter as tk
 from tkinter import ttk
 from tkinter import font as tkfont
 from tkinter import messagebox
 
-from PyMultiDictionary import MultiDictionary
+from PyMultiDictionary import MultiDictionary, DICT_THESAURUS
 from typing import Callable, Tuple, Optional, Dict, Union, List, TYPE_CHECKING, Any
 
 if TYPE_CHECKING:
@@ -41,6 +42,8 @@ class SettingsWindow(object):
     _cfg: '_Settings'
     _dict_langs: Dict[str, str]
     _dict_pipelines: Dict[str, str]
+    _pipeline_descr_title: 'tk.Label'
+    _pipeline_description: 'tk.Label'
     _var_check_repetition: 'tk.BooleanVar'
     _var_check_repetition_stemming: 'tk.BooleanVar'
     _var_check_repetition_stopwords: 'tk.BooleanVar'
@@ -128,10 +131,21 @@ class SettingsWindow(object):
 
         self._var_pipeline = tk.StringVar(self.root)
         self._var_pipeline.set(self._cfg.lang(cfg.get(cfg.CFG_PIPELINE, update=False)))  # default value
+        self._var_pipeline.trace('w', self._change_description_pipeline)
 
         pipe = tk.OptionMenu(f, self._var_pipeline, *list(self._dict_pipelines.keys()))
         pipe.pack(side=tk.LEFT)
 
+        f = tk.Frame(f0, border=0)
+        f.pack(fill='both', pady=0)
+        label_fg = '#999999' if ut.IS_OSX else '#666666'
+        label_pad = 5 if ut.IS_OSX else 15
+        self._pipeline_descr_title = make_label(f, w=70, h=40, side=tk.LEFT, fg=label_fg,  # bg='blue',
+                                                bd=0, relief=tk.SUNKEN, anchor=tk.E, pad=(0, label_pad, 5, 7))
+        self._pipeline_description = make_label(f, w=window_size[0] - 70, h=40, side=tk.LEFT, fg=label_fg,  # bg='red',
+                                                bd=0, relief=tk.SUNKEN, anchor=tk.W, pad=(0, 0, 5, 0), justify=tk.LEFT)
+        self._change_description_pipeline()
+
         # Check repetition
         f_repetition = tk.LabelFrame(f0, text=self._cfg.lang('cfg_words_repetition'), bd=1, relief=tk.GROOVE)
         f_repetition.pack(fill='both')
@@ -239,6 +253,19 @@ class SettingsWindow(object):
         # Update
         self.root.update()
 
+    # noinspection PyUnusedLocal
+    def _change_description_pipeline(self, *args) -> None:
+        """
+        Event raised if pipeline is changed.
+
+        :param args: Args
+        """
+        t = self._cfg.lang(f'{self._dict_pipelines[self._var_pipeline.get()]}_description')
+        # t = f'{self._var_pipeline.get()}: {t}'
+        t = '\n'.join(textwrap.wrap(t, 41 if ut.IS_OSX else 50))
+        self._pipeline_descr_title['text'] = f'{self._var_pipeline.get()}:'
+        self._pipeline_description['text'] = t
+
     def close(self) -> None:
         """
         Close the window.
@@ -413,7 +440,10 @@ class DictionaryGUI(object):
         if lang not in self._dictionary._langs.keys():
             self.close()
         lang_name = self._dictionary.get_language_name(lang, self._cfg.get(self._cfg.CFG_LANG))
-        self.root.title(f'{self._cfg.lang("dictionary")} [{lang_name}]')
+        try:
+            self.root.title(f'{self._cfg.lang("dictionary")} [{lang_name}]')
+        except tk.TclError:
+            pass
         self._lang = lang
         self._update_buttons()
 
@@ -425,10 +455,16 @@ class DictionaryGUI(object):
         :return:
         """
         # Check language capability
-        syn, mean, _, ant = self._dictionary._langs[self._lang]
-        self._syn['state'] = tk.NORMAL if (syn and enable) else tk.DISABLED
-        self._mean['state'] = tk.NORMAL if (mean and enable) else tk.DISABLED
-        self._ant['state'] = tk.NORMAL if (ant and enable) else tk.DISABLED
+        try:
+            syn, mean, _, ant = self._dictionary._langs[self._lang]
+        except KeyError:
+            syn, mean, ant = False, False, False
+        try:
+            self._syn['state'] = tk.NORMAL if (syn and enable) else tk.DISABLED
+            self._mean['state'] = tk.NORMAL if (mean and enable) else tk.DISABLED
+            self._ant['state'] = tk.NORMAL if (ant and enable) else tk.DISABLED
+        except tk.TclError:
+            pass
 
     @staticmethod
     def _process_out_key(event: 'tk.Event') -> Optional['tk.Event']:
@@ -531,7 +567,10 @@ class DictionaryGUI(object):
         key = 'synonym'
 
         def _process(word: str):
-            self._query_output[key] = self._dictionary.synonym(self._lang, word)
+            if self._lang == 'en':
+                self._query_output[key] = self._dictionary.synonym(self._lang, word, DICT_THESAURUS)
+            else:
+                self._query_output[key] = self._dictionary.synonym(self._lang, word)
             self._query_active[key] = 0
             return
 
