@@ -10,9 +10,11 @@ __all__ = ['PyDetexGUI']
 
 import concurrent.futures
 import tkinter as tk
+from tkinter import filedialog
 from tkinter import ttk
 from tkinter import messagebox
 
+import os
 import pyperclip
 import requests
 import string
@@ -102,13 +104,15 @@ class PyDetexGUI(object):
         f0 = tk.Frame(self._root, border=10, width=window_size[0], height=50)
         f0.pack()
         f0.pack_propagate(0)
+        tk.Button(f0, text=ut.button_text(self._cfg.lang('open_file')), command=self._open_file,
+                  relief=tk.GROOVE).pack(side=tk.LEFT)
         tk.Button(f0, text=ut.button_text(self._cfg.lang('about')), command=self._about,
                   relief=tk.GROOVE).pack(side=tk.RIGHT)
         tk.Button(f0, text=ut.button_text(self._cfg.lang('settings')), command=self._open_settings,
-                  relief=tk.GROOVE).pack(side=tk.RIGHT, padx=(0, 7 if ut.IS_OSX else 10))
+                  relief=tk.GROOVE).pack(side=tk.RIGHT, padx=(0, 7 if ut.IS_OSX else 11))
         self._dictionary_btn = tk.Button(f0, text=ut.button_text(self._cfg.lang('dictionary')),
                                          command=self._open_dictionary, relief=tk.GROOVE)
-        self._dictionary_btn.pack(side=tk.RIGHT, padx=(0, 7 if ut.IS_OSX else 10))
+        self._dictionary_btn.pack(side=tk.RIGHT, padx=(0, 7 if ut.IS_OSX else 11))
 
         hthick, hcolor = 3 if ut.IS_OSX else 1, '#426392' if ut.IS_OSX else '#475aff'
         fsize = self._cfg.get(self._cfg.CFG_FONT_SIZE)
@@ -138,10 +142,6 @@ class PyDetexGUI(object):
         self._text_out = gui_ut.RichText(self._cfg, f2, wrap='word', highlightthickness=hthick,
                                          highlightcolor=hcolor, font_size=fsize, copy=True)
         self._text_out.bind('<Key>', self._process_out_key)
-        self._text_out.bind('<Button>', self._check_dictionary_selected)
-        self._text_out.bind('<ButtonRelease>', self._check_dictionary_selected)
-        self._text_out.bind('<FocusIn>', self._check_dictionary_selected)
-        self._text_out.bind('<FocusOut>', self._check_dictionary_selected)
         self._text_out.pack(fill='both')
 
         # ----------------------------------------------------------------------
@@ -220,43 +220,30 @@ class PyDetexGUI(object):
         self._insert_in(self._cfg.lang('placeholder'))
         self._root.update()
 
-    def _get_selected_word(self) -> str:
+    def _open_file(self) -> None:
         """
-        Get selected word.
-
-        :return: Return the current selected word
+        Opens a file.
         """
+        self._status(self._cfg.lang('status_requesting_file'))
+        last_path_cfg = self._cfg.get(self._cfg.CFG_LAST_OPENED_FOLDER)
+        initial_dir = '/' if not os.path.isdir(last_path_cfg) else last_path_cfg
+        filename = filedialog.askopenfilename(
+            title=self._cfg.lang('open_file_select'),
+            initialdir=initial_dir,
+            filetypes=[(self._cfg.lang('open_file_latex_file'), '*.tex')])
+        if filename == '':
+            return self._status_clear()
+        filename_dir = os.path.dirname(filename)
+        self._cfg.set(self._cfg.CFG_LAST_OPENED_FOLDER, filename_dir)
         try:
-            if self._text_in.selection_get() != '' and self._root.focus_get() == self._text_in:
-                sf = self._text_in.count('1.0', 'sel.first')
-                if sf is None:
-                    sf = (0,)
-                word = ut.get_word_from_cursor(self._text_in.get(0.0, tk.END), sf[0])
-                if word != '':
-                    return word
-        except tk.TclError:
+            f = open(filename, 'r')
+            text = ''.join(f.readlines())
+            f.close()
+            self._clear()
+            self._insert_in(text)
+            self._cfg.save()
+        except PermissionError:
             pass
-        try:
-            if self._text_out.selection_get() != '' and self._root.focus_get() == self._text_out:
-                sf = self._text_out.count('1.0', 'sel.first')
-                if sf is None:
-                    sf = (0,)
-                word = ut.get_word_from_cursor(self._text_out.get(0.0, tk.END), sf[0])
-                if word != '':
-                    return word
-        except tk.TclError:
-            pass
-        return ''
-
-    # noinspection PyUnusedLocal
-    def _check_dictionary_selected(self, *args) -> str:
-        """
-        Check dictionary selected word.
-
-        :return: Return the selected word
-        """
-        word = ut.tokenize(self._get_selected_word())
-        return word
 
     def _insert_in(self, text: str) -> None:
         """
@@ -334,9 +321,12 @@ class PyDetexGUI(object):
         :param event: Event
         :return: Event
         """
-        w = self._cfg.get(self._cfg.CFG_WINDOW_SIZE)[0] > 750
-        fout = self._cfg.lang('status_cursor_input_focusout') if w else self._cfg.lang(
-            'status_cursor_input_focusout_min')
+        fout = self._cfg.lang('status_cursor_input_focusout')
+        w = self._cfg.get(self._cfg.CFG_WINDOW_SIZE)[0]
+        if w < 1000:
+            fout = self._cfg.lang('status_cursor_input_focusout_min')
+        if w < 750:
+            fout = self._cfg.lang('status_cursor_input_focusout_min2')
         self._status_bar_cursor['text'] = fout
         self._status_bar_cursor_sel['text'] = ''
         return event
@@ -380,16 +370,14 @@ class PyDetexGUI(object):
         except tk.TclError:
             self._status_bar_cursor_sel['text'] = ''
 
-        self._check_dictionary_selected()
-
-    def _process_out_key(self, event: 'tk.Event') -> Optional['tk.Event']:
+    @staticmethod
+    def _process_out_key(event: 'tk.Event') -> Optional['tk.Event']:
         """
         Process out keys.
 
         :param event: Event
         :return: Event
         """
-        self._check_dictionary_selected()
         # noinspection PyUnresolvedReferences
         if event.char == '':
             return event
@@ -412,7 +400,7 @@ class PyDetexGUI(object):
         # Check if dictionary is available
         # noinspection PyProtectedMember
         if self._detected_lang_tag in self._dictionary._langs.keys():
-            self._status_bar_lang['text'] += f'  [{self._cfg.lang("dictionary")}]'
+            self._status_bar_lang['text'] += f' [{self._cfg.lang("dictionary")}]'
             self._dictionary_btn['state'] = tk.NORMAL
         else:
             self._dictionary_btn['state'] = tk.DISABLED
@@ -424,6 +412,7 @@ class PyDetexGUI(object):
         # noinspection PyProtectedMember
         if self._cfg._last_opened_day_diff >= 7:
             self._root.after(1000, self._check_version_event)
+        self._root.after(100, lambda: self._root.lift())
         self._root.mainloop()
 
     def _clear(self) -> None:
@@ -466,11 +455,14 @@ class PyDetexGUI(object):
         # Font format
         font_format = self._cfg.get(self._cfg.CFG_OUTPUT_FONT_FORMAT)
         PARSER_FONT_FORMAT['cite'] = FONT_TAGS['link'] if font_format else ''
+        PARSER_FONT_FORMAT['equation'] = FONT_TAGS['italic'] if font_format else ''
         PARSER_FONT_FORMAT['normal'] = FONT_TAGS['normal'] if font_format else ''
         PARSER_FONT_FORMAT['ref'] = FONT_TAGS['link'] if font_format else ''
+        PARSER_FONT_FORMAT['tex_text_tag'] = FONT_TAGS['bold'] if font_format else ''
+        PARSER_FONT_FORMAT['tex_text_tag_content'] = FONT_TAGS['italic'] if font_format else ''
 
         # Process the text and get the language
-        out = self.pipeline(text)
+        out = self.pipeline(text, self._detected_lang_tag)
         words = len(self._tokenizer.tokenize(out))
         self._cfg.add_words(words)
 
@@ -556,7 +548,8 @@ class PyDetexGUI(object):
         if self._settings_window:
             self._settings_window.root.lift()
             return
-        self._settings_window = gui_ut.SettingsWindow((375, 427 if ut.IS_OSX else 448), self._cfg)
+        delta_h = 0 if ut.IS_OSX else 21
+        self._settings_window = gui_ut.SettingsWindow((375, 465 + delta_h), self._cfg)
         self._settings_window.on_destroy = self._close_settings
         try:
             # self._settings_window.root.mainloop(1)
@@ -573,11 +566,49 @@ class PyDetexGUI(object):
         if self._ready:
             self._process()
 
+    def _get_selected_word(self) -> str:
+        """
+        Get selected word.
+
+        :return: Return the current selected word
+        """
+        word = ''
+
+        # Text in
+        try:
+            if self._text_in.selection_get() != '' and self._root.focus_get() == self._text_in:
+                text = self._text_in.get(0.0, tk.END)
+                sf = self._text_in.count('1.0', 'sel.first')
+                se = self._text_in.count('1.0', 'sel.last')
+                if sf is None:
+                    sf = (0,)
+                # print([text[sf[0]:se[0]]])
+                word = ut.get_phrase_from_cursor(text, sf[0], se[0] - 1)
+        except tk.TclError:
+            pass
+
+        # Text out
+        try:
+            if self._text_out.selection_get() != '' and self._root.focus_get() == self._text_out and word == '':
+                text = self._text_out.get(0.0, tk.END)
+                sf = self._text_out.count('1.0', 'sel.first')
+                se = self._text_out.count('1.0', 'sel.last')
+                if sf is None:
+                    sf = (0,)
+                word = ut.get_phrase_from_cursor(text, sf[0], se[0] - 1)
+        except tk.TclError:
+            pass
+
+        # Apply pipeline
+        word = pip.strict(word)
+        word = ut.tokenize(word)
+        return word
+
     def _open_dictionary(self) -> None:
         """
         Launch dictionary.
         """
-        selected_word = self._check_dictionary_selected()
+        selected_word = self._get_selected_word()
         if self._dictionary_window:
             if selected_word != '':
                 self._dictionary_window.insert_word(selected_word)
