@@ -1,6 +1,6 @@
 """
 PyDetex
-https://github.com/ppizarror/pydetex
+https://github.com/ppizarror/PyDetex
 
 PARSERS
 Defines parsers, which perform a single task for removal LaTex things.
@@ -9,7 +9,9 @@ Defines parsers, which perform a single task for removal LaTex things.
 __all__ = [
     'find_str',
     'FONT_FORMAT_SETTINGS',
+    'process_chars_equations',
     'process_cite',
+    'process_cite_replace_tags',
     'process_inputs',
     'process_labels',
     'process_quotes',
@@ -24,21 +26,95 @@ __all__ = [
 ]
 
 import os
+import pydetex.utils as ut
+
 from typing import List, Tuple, Union
 
 # Files
 _NOT_FOUND_FILES = []
 _PRINT_LOCATION = False
-_TAG_FILE_ERROR = '|FILEERROR|'
+_TAG_FILE_ERROR = '⇱FILEERROR⇲'
+_TAG_OPEN_CITE = '⇱OPEN_CITE⇲'
+_TAG_CLOSE_CITE = '⇱CLOSE_CITE⇲'
 
-# Parser font format
+# Parser font format. This dict stores the font of some tex elements to be represented
+# in the GUI text editor. The values are the same of _fonts.FONT_TAGS. By default
+# they are empty, and are updated in the PyDetexGUI._process() method
 FONT_FORMAT_SETTINGS = {
     'cite': '',
+    'equation': '',
     'normal': '',
-    'ref': ''
+    'ref': '',
+    'tex_text_tag': '',
+    'tex_text_tag_content': ''
 }
 
-import pydetex.utils as ut
+LANG_TEX_TEXT_TAGS = ut.LangTexTextTags()
+
+# Replace symbols
+REPLACE_SYMBOLS_LIBRARY: List[Tuple[str, str]] = [
+    # Common
+    ('\\item', '-'),
+    ('--', '–'),
+    ('\\\\', '\n'),
+    ('\\ ', ' '),
+
+    # Letters
+    ('ﬁ', 'fi'),
+]
+REPLACE_UNIQUE_SYMBOLS_LIBRARY: List[Tuple[str, str]] = [
+    # Greek
+    ('\\alpha', 'α'),
+    ('\\beta', 'β'),
+    ('\\chi', 'χ'),
+    ('\\delta', 'δ'),
+    ('\\Delta', 'Δ'),
+    ('\\epsilon', 'ϵ'),
+    ('\\eta', 'η'),
+    ('\\gamma', 'γ'),
+    ('\\Gamma', 'Γ'),
+    ('\\iota', 'ι'),
+    ('\\kappa', 'κ'),
+    ('\\lambda', 'λ'),
+    ('\\Lambda', 'Λ'),
+    ('\\mu', 'μ'),
+    ('\\nu', 'ν'),
+    ('\\omega', 'ω'),
+    ('\\Omega', 'Ω'),
+    ('\\phi', 'φ'),
+    ('\\Phi', 'Φ'),
+    ('\\pi', 'π'),
+    ('\\Pi', 'Π'),
+    ('\\psi', 'ψ'),
+    ('\\Psi', 'Ψ'),
+    ('\\rho', 'ρ'),
+    ('\\sigma', 'σ'),
+    ('\\Sigma', 'Σ'),
+    ('\\tau', 'τ'),
+    ('\\theta', 'θ'),
+    ('\\Theta', 'Θ'),
+    ('\\upsilon', 'υ'),
+    ('\\varepsilon', 'ε'),
+    ('\\varphi', 'φ'),
+    ('\\varrho', 'ϱ'),
+    ('\\vartheta', '𝜗'),
+    ('\\xi', 'ξ'),
+    ('\\Xi', 'Ξ'),
+    ('\\zeta', 'ζ'),
+
+    # Arrows
+    ('\\leftarrow', '←'),
+    ('\\rightarrow', '→'),
+    ('\\Leftarrow', '⇐'),
+    ('\\Rightarrow', '⇒'),
+    ('\\uparrow', '↑'),
+    ('\\downarrow', '↓'),
+    ('\\Uparrow', '⇑'),
+    ('\\Downarrow', '⇓'),
+    ('\\leftrightarrow', '↔'),
+    ('\\longleftarrow', '⟵'),
+    ('\\longrightarrow', '⟶')
+]
 
 
 def _find_str(s: str, char: str) -> int:
@@ -63,6 +139,37 @@ def _find_str(s: str, char: str) -> int:
     return -1
 
 
+def _os_listfolder() -> List[str]:
+    """
+    Returns the folders from the current path.
+
+    :return: Folder list
+    """
+    dirs = os.listdir('./')
+    folders = []
+    for k in dirs:
+        if os.path.isdir(k):
+            folders.append(k + '/')
+    return folders
+
+
+def _load_file(f: str, path: str) -> str:
+    """
+    Try to load a file.
+
+    :param f: Filename
+    :param path: Path to look from
+    :return: File contents
+    """
+    try:
+        fo = open(path + f, 'r')
+        s = '\n'.join(fo.readlines())
+        fo.close()
+        return s
+    except FileNotFoundError:
+        return _TAG_FILE_ERROR
+
+
 def find_str(s: str, char: Union[str, List[str], Tuple[str, ...]]) -> int:
     """
     Finds a sequence within a string, and returns the position. If not exists, returns -1.
@@ -167,17 +274,31 @@ def process_cite(s: str) -> str:
                     if w not in cites.keys():
                         cites[w] = len(cites.keys()) + 1
                     c = c.replace(w, str(cites[w]))
-                s = s[:k] + FONT_FORMAT_SETTINGS['cite'] + '[' + c + ']' + FONT_FORMAT_SETTINGS['normal'] + s[
-                                                                                                            k + j + 1:]
+                s = s[:k] + FONT_FORMAT_SETTINGS['cite'] + _TAG_OPEN_CITE + c + \
+                    _TAG_CLOSE_CITE + FONT_FORMAT_SETTINGS['normal'] + s[k + j + 1:]
                 break
 
 
+def process_cite_replace_tags(s: str, cite_format: Tuple[str, str] = ('[', ']')) -> str:
+    """
+    Replaces cite tags to an specific format.
+
+    :param s: String
+    :param cite_format: Cite format
+    :return: String with no cites
+    """
+    assert len(cite_format) == 2
+    s = s.replace(_TAG_OPEN_CITE, (cite_format[0]))
+    s = s.replace(_TAG_CLOSE_CITE, (cite_format[1]))
+    return s
+
+
 def process_labels(s: str) -> str:
     """
     Removes labels.
 
     :param s: String
-    :return:
+    :return: String with no labels
     """
     while True:
         k = find_str(s, '\\label{')
@@ -235,22 +356,27 @@ def remove_comments(s: str) -> str:
     :param s: Text
     :return: Text without comments
     """
-    symbol = '|COMMENTPERCENTAGESYMBOL|'
+    comment_symbol = '⇱COMMENTPERCENTAGESYMBOL⇲'
+    newline_symbol = '⇱NEWLINESYMBOL⇲'
     s = s.replace('  ', ' ')
-    s = s.replace('\\%', symbol)
+    s = s.replace('\\\\', newline_symbol)
+    s = s.replace('\\%', comment_symbol)
     k = s.split('\n')
     for r in range(len(k)):
         k[r] = k[r].strip()  # Strips all text
-    line_merge = []
+    line_merge: List[bool] = []
     for r in range(len(k)):
         sp = k[r].split('%')
         k[r] = sp[0]  # Removes all comments from list
         line_merge.append(len(sp) > 1)
     line_merge.append(False)
+    line_merge2: List[bool] = line_merge.copy()
     k.append('')
     for r in range(len(k)):
         if line_merge[r] and not line_merge[r + 1] and k[r + 1] != '':
-            line_merge[r + 1] = True
+            line_merge2[r + 1] = True
+    for r in range(len(k)):
+        line_merge[r] = line_merge[r] or line_merge2[r]
     new_k = []
     j = 0
     merged_str = ''
@@ -279,7 +405,8 @@ def remove_comments(s: str) -> str:
     if len(w) > 0 and w[-1] == '':  # Removes last space
         w.pop()
     s = '\n'.join(w)
-    s = s.replace(symbol, '%').strip()
+    s = s.replace(comment_symbol, '%').strip()
+    s = s.replace(newline_symbol, '\\\\')
     return s
 
 
@@ -290,104 +417,26 @@ def simple_replace(s: str) -> str:
     :param s: String
     :return: String with replaced items
     """
-    library: List[Tuple[str, str]] = [
-        # Common
-        ('\\item', '-'),
-        ('--', '–'),
-        ('\\\\', '\n'),
-        ('\\ ', ' '),
-
-        # Letters
-        ('ﬁ', 'fi'),
-
-        # Greek
-        ('\\alpha', 'α'),
-        ('\\beta', 'β'),
-        ('\\chi', 'χ'),
-        ('\\delta', 'δ'),
-        ('\\Delta', 'Δ'),
-        ('\\epsilon', 'ϵ'),
-        ('\\eta', 'η'),
-        ('\\gamma', 'γ'),
-        ('\\Gamma', 'Γ'),
-        ('\\iota', 'ι'),
-        ('\\kappa', 'κ'),
-        ('\\lambda', 'λ'),
-        ('\\Lambda', 'Λ'),
-        ('\\mu', 'μ'),
-        ('\\nu', 'ν'),
-        ('\\omega', 'ω'),
-        ('\\Omega', 'Ω'),
-        ('\\phi', 'φ'),
-        ('\\Phi', 'Φ'),
-        ('\\pi', 'π'),
-        ('\\Pi', 'Π'),
-        ('\\psi', 'ψ'),
-        ('\\Psi', 'Ψ'),
-        ('\\rho', 'ρ'),
-        ('\\sigma', 'σ'),
-        ('\\Sigma', 'Σ'),
-        ('\\tau', 'τ'),
-        ('\\theta', 'θ'),
-        ('\\Theta', 'Θ'),
-        ('\\upsilon', 'υ'),
-        ('\\varepsilon', 'ε'),
-        ('\\varphi', 'φ'),
-        ('\\varrho', 'ϱ'),
-        ('\\vartheta', '𝜗'),
-        ('\\xi', 'ξ'),
-        ('\\Xi', 'Ξ'),
-        ('\\zeta', 'ζ'),
-
-        # Arrows
-        ('\\leftarrow', '←'),
-        ('\\rightarrow', '→'),
-        ('\\Leftarrow', '⇐'),
-        ('\\Rightarrow', '⇒'),
-        ('\\uparrow', '↑'),
-        ('\\downarrow', '↓'),
-        ('\\Uparrow', '⇑'),
-        ('\\Downarrow', '⇓'),
-        ('\\leftrightarrow', '↔'),
-        ('\\longleftarrow', '⟵'),
-        ('\\longrightarrow', '⟶')
-    ]
-    for w in library:
+    for w in REPLACE_SYMBOLS_LIBRARY:
         s = s.replace(w[0], w[1])
+
+    # Replace unique symbols
+    s += ' '
+    invalid_tag = '⇱SYMBOL_REPLACE_TAG_TOKEN⇲'
+    for w in REPLACE_UNIQUE_SYMBOLS_LIBRARY:
+        word, repl = w
+        while True:
+            k = s.find(word)
+            if k == -1:
+                break
+            if s[k + len(word)] not in ut.VALID_TEX_COMMAND_CHARS:
+                s = s[0:k] + repl + s[k + len(word):]
+            else:
+                s = s[0:k + 1] + invalid_tag + s[k + 1:]
+    s = s[0:len(s) - 1].replace(invalid_tag, '')
     return s
 
 
-def _load_file(f: str, path: str) -> str:
-    """
-    Try to load a file.
-
-    :param f: Filename
-    :param path: Path to look from
-    :return: File contents
-    """
-    try:
-        fo = open(path + f, 'r')
-        s = '\n'.join(fo.readlines())
-        fo.close()
-        return s
-    except FileNotFoundError:
-        return _TAG_FILE_ERROR
-
-
-def _os_listfolder() -> List[str]:
-    """
-    Returns the folders from the current path.
-
-    :return: Folder list
-    """
-    dirs = os.listdir('./')
-    folders = []
-    for k in dirs:
-        if os.path.isdir(k):
-            folders.append(k + '/')
-    return folders
-
-
 def process_inputs(s: str) -> str:
     """
     Process inputs, and try to copy the content.
@@ -396,11 +445,12 @@ def process_inputs(s: str) -> str:
     :return: Text copied with data
     """
     global _PRINT_LOCATION, _NOT_FOUND_FILES
+    symbol = '⇱INPUTFILETAG⇲'
     tx = ''
     while True:
         k = find_str(s, '\\input{')
         if k == -1:
-            return s.replace('|INPUTFILETAG', '\\input{')
+            return s.replace(symbol, '\\input{')
         m = 0
         for j in range(len(s)):
             if s[k + j] == '{':
@@ -427,12 +477,12 @@ def process_inputs(s: str) -> str:
                             break
                     if tx == _TAG_FILE_ERROR:
                         _NOT_FOUND_FILES.append(tex_file)
-                        s = s[:k] + '|INPUTFILETAG' + s[k + m + 1:]
+                        s = s[:k] + symbol + s[k + m + 1:]
                     else:
                         print('\tFile found and loaded')
                         s = s[:k] + tx + s[k + j + 1:]
                 else:
-                    s = s[:k] + '|INPUTFILETAG' + s[k + m + 1:]
+                    s = s[:k] + symbol + s[k + m + 1:]
                 break
 
 
@@ -465,11 +515,53 @@ def remove_commands_char(s: str, chars: Union[Tuple[str, str], str], ignore_esca
     return new_s
 
 
-def remove_commands_param(s: str) -> str:
+def output_text_for_some_commands(s: str, lang: str) -> str:
+    """
+    Replaces the command for a particular text.
+
+    :param s: Latex string code
+    :param lang: Language tag of the code
+    :return: Text string or empty if error
+    """
+    # Stores the commands to be transformed
+    # (command name, argument number, argument is optional, LANG_TEX_TAGS tag to be replaced, total commands)
+    commands = [
+        ('caption', 1, False, 'caption', 1),
+        ('insertimage', 3, False, 'figure_caption', 3),  # Format \insertimage{file}{args}{caption}
+        ('insertimage', 4, False, 'figure_caption', 4),  # Format \insertimage[opt. keywords]{file}{args}{caption}
+        ('insertimageboxed', 4, False, 'figure_caption', 4),
+        ('insertimageboxed', 5, False, 'figure_caption', 5),
+        ('subfloat', 1, True, 'sub_figure_title', 1)
+    ]
+    new_s = ''
+
+    # Get the commands
+    cmd_args = ut.get_tex_commands_args(s)
+    for c in cmd_args:
+        for cmd in commands:
+            if c[0] == cmd[0]:
+                _, cmd_argnum, cmd_is_optional, cmd_tag, total_commands = cmd
+                if len(c) - 1 >= cmd_argnum and len(c) - 1 == total_commands:
+                    if c[cmd_argnum][1] == cmd_is_optional:
+                        argv = c[cmd_argnum][0].replace('\n', ' ')  # Command's argument to process
+                        argv = remove_commands_param(argv, lang)  # Remove commands within the argument
+                        argv = argv.strip()
+                        if argv != '':
+                            argv = FONT_FORMAT_SETTINGS['tex_text_tag_content'] + argv  # Add format text
+                            text = LANG_TEX_TEXT_TAGS.get(lang, cmd_tag).format(argv)
+                            new_s += FONT_FORMAT_SETTINGS['tex_text_tag'] + text + FONT_FORMAT_SETTINGS['normal']
+
+                        break
+
+    return new_s
+
+
+def remove_commands_param(s: str, lang: str) -> str:
     """
     Remove all commands with params.
 
     :param s: String
+    :param lang: Language tag of the code
     :return: Code with removed chars
     """
     tex_tags = ut.find_tex_commands(s)
@@ -485,10 +577,28 @@ def remove_commands_param(s: str) -> str:
             elif i < tex_tags[k][3] + 1:
                 pass
             else:  # advance to other tag
+                sub_s = s[tex_tags[k][0]:tex_tags[k][3] + 2]
+                if not tex_tags[k][4]:  # If the command does not continue
+                    new_s += output_text_for_some_commands(sub_s, lang)
                 k += 1
         else:
             new_s += s[i]
 
+    # Replace all command symbols
+    parenthesis_open_symbol = '⇱PARENTHESIS_OPEN_SYMBOL⇲'
+    parenthesis_close_symbol = '⇱PARENTHESIS_CLOSE_SYMBOL⇲'
+    parenthesis_sq_open_symbol = '⇱PARENTHESIS_SQ_OPEN_SYMBOL⇲'
+    parenthesis_sq_close_symbol = '⇱PARENTHESIS_SQ_CLOSE_SYMBOL⇲'
+    new_s = new_s.replace('\\{', parenthesis_open_symbol)
+    new_s = new_s.replace('\\}', parenthesis_close_symbol)
+    new_s = new_s.replace('\\[', parenthesis_sq_open_symbol)
+    new_s = new_s.replace('\\]', parenthesis_sq_close_symbol)
+    new_s = new_s.replace('{', '').replace('}', '')  # .replace('[', '').replace(']', '')
+    new_s = new_s.replace(parenthesis_open_symbol, '\\{')
+    new_s = new_s.replace(parenthesis_close_symbol, '\\}')
+    new_s = new_s.replace(parenthesis_sq_open_symbol, '\\[')
+    new_s = new_s.replace(parenthesis_sq_close_symbol, '\\]')
+
     return new_s
 
 
@@ -517,3 +627,41 @@ def remove_commands_param_noargv(s: str) -> str:
             new_s += s[i]
 
     return new_s
+
+
+def process_chars_equations(s: str, lang: str, single_only: bool) -> str:
+    """
+    Process single char equations, removing the $ symbols.
+
+    :param s: Latex code
+    :param lang: Language tag of the code
+    :param single_only: Only process single char equations. If False, replaces the equation by a text-label
+    :return: Code without symbols
+    """
+    tex_tags = ut.find_tex_command_char(s, '$', True)
+    if len(tex_tags) == 0:
+        return s
+    new_s = ''
+    k = 0  # Moves through tags
+    eqn_number = 0
+
+    for i in range(len(s)):
+        if k < len(tex_tags):
+            if i < tex_tags[k][0]:
+                new_s += s[i]
+            elif i < tex_tags[k][1]:
+                if i == tex_tags[k][0] + 1 and tex_tags[k][1] - tex_tags[k][0] == 2:
+                    new_s += FONT_FORMAT_SETTINGS['equation'] + s[i] + FONT_FORMAT_SETTINGS['normal']
+            else:  # advance to other tag
+                if tex_tags[k][1] - tex_tags[k][0] > 2:
+                    if not single_only:
+                        new_s += LANG_TEX_TEXT_TAGS.get(lang, 'multi_char_equ').format(eqn_number)
+                        eqn_number += 1
+                    else:
+                        new_s += s[tex_tags[k][0]:tex_tags[k][1] + 1]
+                k += 1
+
+        else:
+            new_s += s[i]
+
+    return new_s
