@@ -12,6 +12,7 @@ __all__ = [
     'apply_tag_tex_commands_no_argv',
     'Button',
     'check_repeated_words',
+    'complete_langs_dict',
     'detect_language',
     'find_tex_command_char',
     'find_tex_commands',
@@ -20,13 +21,16 @@ __all__ = [
     'get_diff_startend_word',
     'get_language_name',
     'get_number_of_day',
+    'get_tex_commands_args',
     'get_word_from_cursor',
     'IS_OSX',
+    'LangTexTextTags',
     'make_stemmer',
     'RESOURCES_PATH',
     'split_tags',
     'syntax_highlight',
     'tokenize',
+    'VALID_TEX_COMMAND_CHARS',
     'validate_float',
     'validate_int'
 ]
@@ -113,7 +117,7 @@ def button_text(s: str) -> str:
     :param s: Button's text
     :return: Text
     """
-    return s if IS_OSX else '  {0}  '.format(s)
+    return s if IS_OSX else f'  {s}  '
 
 
 def validate_int(p: str) -> bool:
